@@ -24,7 +24,7 @@ def AssembleMatrix(Mesh, ElementData, ProblemData, ModelData, MatrixType):
     A_e   = zeros(n, dtype='float64')
     A_int = zeros(n, dtype='float64')
     # Obtiene pesos y posiciones de la Cuadratura de Gauss
-    gp = CuadraturaGauss(ElementData.noInt, ProblemData.SpaceDim)
+    gp = GaussianQuadrature(ElementData.noInt, ProblemData.SpaceDim)
     # Bucle para ensamblar la matriz de cada elemento
     for connect_element in Mesh.Conex:
         # Obtiene coordenadas de nodos del elemento
@@ -32,7 +32,7 @@ def AssembleMatrix(Mesh, ElementData, ProblemData, ModelData, MatrixType):
         # Bucle para realizar la integración según Cuadratura de Gauss
         for gauss_point in gp:
             # Evalua los puntos de Gauss en las Funciones de Forma
-            [N, dN,ddN, j] = FunciónForma(x_element, gauss_point, ElementData.type)
+            [N, dN,ddN, j] = ShapeFunction(x_element, gauss_point, ElementData.type)
             dX = gauss_point[0]*j
             # Obtiene la Matriz de cada Elemento según el Problema(Elasticidad, Timoshenko, Bernoulli, etc)
             A_int = eval(ProblemData.pde +'(A_int, x_element, N, dN,ddN, ProblemData,ElementData, ModelData, dX, MatrixType)')
@@ -68,7 +68,7 @@ def AssembleVector(Mesh, ElementData, ProblemData, ModelData, MatrixType):
     f_e    = zeros(n,'float64')
     f_int  = zeros(n,'float64')
     # Obtiene pesos y posiciones de la Cuadratura de Gauss
-    gp = CuadraturaGauss(ElementData.noInt,ProblemData.SpaceDim)
+    gp = GaussianQuadrature(ElementData.noInt,ProblemData.SpaceDim)
     # Bucle para ensamblar la matriz de cada elemento
     for connect_element in Mesh.Conex:
         # Obtiene coordenadas de nodos del elemento
@@ -76,7 +76,7 @@ def AssembleVector(Mesh, ElementData, ProblemData, ModelData, MatrixType):
         # Bucle para realizar la integración según Cuadratura de Gauss
         for gauss_point in gp:
             # Evalua los puntos de Gauss en las Funciones de Forma
-            [N, dN,ddN, j] = FunciónForma(x_element, gauss_point,ElementData.type)
+            [N, dN,ddN, j] = ShapeFunction(x_element, gauss_point,ElementData.type)
             dX = gauss_point[0]*j
             # Obtiene la Matriz de cada Elemento según el Problema(Elasticidad, Timoshenko, Bernoulli, etc)
             f_int = eval(ProblemData.pde +'(f_int, x_element, N, dN, ddN, ProblemData, ElementData, ModelData, dX, "VectorF")')
@@ -105,12 +105,12 @@ def ApplyBC(A, f, BC_data, Mesh, ElementData,ProblemData, ModelData,showBC=False
     '''
     for bc in BC_data:
         if int(bc[1]) == 0:  # Neumann
-            dof = int(ElementData.dof*bc[0]+bc[2])-1
+            dof = int(ElementData.dof*bc[0] + bc[2]-1)
             if showBC==True: print("CB Neumann, DOF:",dof)
             if A[dof,dof] == 1.0: continue # ALREADY is Dirichlet BC
             f[dof] = f[dof] + bc[3]
         elif int(bc[1]) == 1:# Dirichlet
-            dof = int(ElementData.dof*bc[0]+bc[2])-1
+            dof = int(ElementData.dof*bc[0] + bc[2]-1)
             if showBC==True: print("CB Dirichlet, DOF:",dof)
             A[dof,:]  = 0.0
             A[dof,dof] = 1.0
@@ -121,7 +121,7 @@ def ApplyBC(A, f, BC_data, Mesh, ElementData,ProblemData, ModelData,showBC=False
 
 
 # Funciones MEF
-def Elasticidad(A, X, N, dN,dNN, ProblemData, ElementData, ModelData, dX, tipo):
+def Elasticity(A, X, N, dN,dNN, ProblemData, ElementData, ModelData, dX, tipo):
     '''Función que retorna la matriz de un EF evaluado en un Punto de Gauss.
 
     Input:
@@ -138,15 +138,38 @@ def Elasticidad(A, X, N, dN,dNN, ProblemData, ElementData, ModelData, dX, tipo):
     Output:
         - A:    Arreglo del elemento finito que se quiere obtener.
     '''
-    n, m, t = ElementData.nodes, ElementData.dof, ModelData.thickness
+    n, m = ElementData.nodes, ElementData.dof
     #
-    if tipo == 'MatrizK':
-        E,v = ModelData.E, ModelData.v # Estado plano de esfuerzos
-        if ModelData.state == 'PlainStress': pass
-        elif ModelData.state == 'PlainStrain': E,v = E/(1.0-v*2),v/(1-v)
-        else: print('El estado plano solo puede ser "PlainStress" o "PlainStrain"')
-        
-        if ProblemData.SpaceDim == 2:
+    if ProblemData.SpaceDim == 1:
+        area = ModelData.area
+        if tipo == 'MatrizK':
+            E = ModelData.E
+            # Formando Matriz B
+            B = zeros((1,m*n))
+            for i in range(m):
+                B[i, i::m] = dN[i]
+            A = B.T@B*dX*E*area
+
+        elif tipo == 'VectorF':
+            # Formando Matriz F
+            Nmat=zeros((m,m*n))
+            for i in range(m):
+                Nmat[i, i::m] = N
+                f = ModelData.selfweight*ModelData.gravity
+                A = Nmat*f*area*dX
+
+        else:
+            print("Debes programar para el tipo %s aún."%tipo)
+
+    elif ProblemData.SpaceDim == 2:
+        t = ModelData.thickness
+
+        if tipo == 'MatrizK':
+            E,v = ModelData.E, ModelData.v # Estado plano de esfuerzos
+            if ModelData.state == 'PlaneStress': pass
+            elif ModelData.state == 'PlaneStrain': E,v = E/(1.0-v*2),v/(1-v)
+            else: print('El estado plano solo puede ser "PlaneStress" o "PlaneStrain"')
+            
             # Formando Matriz D
             D = zeros((3,3))
             D[0,0], D[1,1], D[0,1], D[1,0]= 1.0, 1.0, v, v
@@ -159,46 +182,84 @@ def Elasticidad(A, X, N, dN,dNN, ProblemData, ElementData, ModelData, dX, tipo):
                 B[i, i::m] = dN[i]
             B[2, 0::m] = dN[1]
             B[2, 1::m] = dN[0]
+            #
+            A = B.T@D@B*dX*t
+        #
+        elif tipo == 'MasaConsistente':
+            Nmat = zeros((m, m*n))
+            rho = ModelData.density
+            for i in range(m):
+                Nmat[i, i::m] = N[:]
+            #
+            A = rho*Nmat.T@Nmat*dX*t
+        #
+        elif tipo == 'MasaConcentrada':
+            Nmat = zeros((m, m*n))
+            rho = ModelData.density
+            for i in range(m):
+                Nmat[i, i::m] = N[:]
+            #
+            B = rho*Nmat.T@Nmat*dX*t
+            one = zeros(m*n) + 1.0
+            B = B@one
+            A = zeros((m*n,m*n))
+            # Concentrando Masas
+            for i in range(m*n):
+                A[i,i] = B[i]
+        #
+        elif tipo == 'VectorF':
+            # Formando Matriz F
+            Nmat=zeros((m,m*n))
+            for i in range(m):
+                Nmat[i, i::m] = N[:].T
+                f = ModelData.selfweight*ModelData.gravity[0:m]
+                A = Nmat.T@f*dX*t
+        #
         else:
-            print("Debes programar para %sD aún"%ProblemData.SpaceDim)
-        #
-        A = B.T@D@B*dX*t
-    #
-    elif tipo == 'MasaConsistente':
-        Nmat = zeros((m, m*n))
-        rho = ModelData.density
-        for i in range(m):
-            Nmat[i, i::m] = N[:]
-        #
-        A = rho*Nmat.T@Nmat*dX*t
-    #
-    elif tipo == 'MasaConcentrada':
-        Nmat = zeros((m, m*n))
-        rho = ModelData.density
-        for i in range(m):
-            Nmat[i, i::m] = N[:]
-        #
-        B = rho*Nmat.T@Nmat*dX*t
-        one = zeros(m*n) + 1.0
-        B = B@one
-        A = zeros((m*n,m*n))
-        # Concentrando Masas
-        for i in range(m*n):
-            A[i,i] = B[i]
-    #
-    elif tipo == 'VectorF':
-        # Formando Matriz F
-        Nmat=zeros((m,m*n))
-        for i in range(m):
-            Nmat[i, i::m] = N[:].T
-            f = ModelData.selfweight*ModelData.gravity[0:m]
-            A = Nmat.T@f*dX*t
-    #
+            print("Debes programar para el tipo %s aún"%tipo)
+
+    elif ProblemData.SpaceDim == 3:
+        if tipo == 'MatrizK':
+            E, v = ModelData.E, ModelData.v
+            # Formando Matriz D
+            D = zeros((6, 6))
+            λ = E * v / ((1.0 + v) * (1.0 - 2.0 * v))
+            μ = E / (2.0 * (1.0 + v))
+
+            D[0, 0] = D[1, 1] = D[2, 2] = λ + 2*μ 
+            D[3, 3] = D[4, 4] = D[5, 5] = μ
+            D[0, 1] = D[1, 0] = D[0, 2] = D[2, 0] = D[1, 2] = D[2, 1] = λ
+
+            # Formando Matriz B
+            B = zeros((6, m * n))
+
+            for i in range(m):
+                B[i, i::m] = dN[i]
+
+            B[3, 0::m] = dN[1]
+            B[3, 1::m] = dN[0]
+            B[4, 1::m] = dN[2]
+            B[4, 2::m] = dN[1]
+            B[5, 0::m] = dN[2]
+            B[5, 2::m] = dN[0]
+
+            A = B.T@D@B *dX
+
+        elif tipo == 'VectorF':
+            # Formando Matriz F
+            Nmat=zeros((m,m*n))
+            for i in range(m):
+                Nmat[i, i::m] = N[:].T
+                f = ModelData.selfweight*ModelData.gravity[0:m]
+                A = Nmat.T@f*dX # *t
+                
+        else:
+            print("Debes programar para el tipo %s aún"%tipo)
     else:
-        print("Debes programar para el tipo %s aún"%tipo)
+        print("Debes programar para %sD aún"%ProblemData.SpaceDim)
     return A
 
-def CuadraturaGauss(puntos, dim):
+def GaussianQuadrature(puntos, dim):
     '''
     Esta función define los puntos de integración según la \
     cuadratura de Gauss
@@ -218,7 +279,8 @@ def CuadraturaGauss(puntos, dim):
         elif puntos == 2:	# Integración de Gauss de 2 puntos en 1D
             gp     = zeros((2,2))
             gp[0,:] = 1.0
-            gp[1,0], gp[1,1] =  -1/3**0.5, 1/3**0.5
+            a = 0.5773502691896257
+            gp[1,0], gp[1,1] =  -a, a
             #
         elif puntos == 4: # Integración de Gauss de 4 puntos en 1D
             gp     = zeros((2,4))
@@ -237,16 +299,33 @@ def CuadraturaGauss(puntos, dim):
         elif puntos == 4:## Integración de Gauss de 4 puntos para Quad4
             gp     = zeros((3,4))
             gp[0,:] = 1.0
-            gp[1,0], gp[1,1], gp[1,2], gp[1,3] = -1.0/3**0.5, 1.0/3**0.5, 1.0/3**0.5,-1.0/3**0.5
-            gp[2,0], gp[2,1], gp[2,2], gp[2,3] = -1.0/3**0.5,-1.0/3**0.5, 1.0/3**0.5, 1.0/3**0.5
+            a = 0.5773502691896257
+            gp[1,0], gp[1,1], gp[1,2], gp[1,3] = -a, a, a,-a
+            gp[2,0], gp[2,1], gp[2,2], gp[2,3] = -a,-a, a, a
         else:
             print("Debes programar para %s puntos aún"%puntos)
+    elif dim == 3:
+
+        if puntos == 8:
+            a = 3**-0.5#0.5773502691896257
+
+            gp = zeros((4, 8))
+            gp[0, :] = 1.0
+            gp[1, 0], gp[1, 1], gp[1, 2], gp[1, 3], gp[1, 4], gp[1, 5], gp[
+                1, 6], gp[1, 7] = -a, a, a, -a, -a, a, a, -a
+            gp[2, 0], gp[2, 1], gp[2, 2], gp[2, 3], gp[2, 4], gp[2, 5], gp[
+                2, 6], gp[2, 7] = -a, -a, a, a, -a, -a, a, a
+            gp[3, 0], gp[3, 1], gp[3, 2], gp[3, 3], gp[3, 4], gp[3, 5], gp[
+                3, 6], gp[3, 7] = -a, -a, -a, -a, a, a, a, a
+        else:
+            print("Debes programar para %s puntos aún." %puntos)
+
     else:
-        print("Debes programar para %sD aún"%dim)
+        print("Debes programar para %sD aún."%dim)
     
     return gp.T
 
-def FunciónForma(X,gp,tipo):
+def ShapeFunction(X,gp,tipo):
     '''
     Esta función define funciones de forma y sus derivadas \
     en coordenadas naturales para un elemento finito de \
@@ -308,19 +387,35 @@ def FunciónForma(X,gp,tipo):
         dN[0] = 0.25*a[:]*(1 + b[:]*η) # dN,ξ = 0.25ξi(1+ηiη)
         dN[1] = 0.25*b[:]*(1 + a[:]*ξ) # dN,η = 0.25ηi(1+ξiξ)
         #
+    elif tipo == 'Brick8':
+
+        N, dN, J = zeros((3, 8)), zeros((3, 8)), zeros((3, 3))
+        a = array([-1.0, 1.0, 1.0, -1.0, -1.0, 1.0, 1.0, -1.0])
+        b = array([-1.0, -1.0, 1.0, 1.0, -1.0, -1.0, 1.0, 1.0])
+        c = array([-1.0, -1.0, -1.0, -1.0, 1.0, 1.0, 1.0, 1.0])
+
+        ξ = gp[1]
+        η = gp[2]
+        ζ = gp[3]
+
+        N = 0.125 * (1.0 + a[:] * ξ) * (1.0 + b[:] * η) * (1.0 + c[:] * ζ)
+
+        dN[0] = 0.125 * a[:] * (1 + b[:] * η) * (1.0 + c[:] * ζ)
+        dN[1] = 0.125 * b[:] * (1 + a[:] * ξ) * (1.0 + c[:] * ζ)
+        dN[2] = 0.125 * c[:] * (1 + a[:] * ξ) * (1.0 + b[:] * η)
+
     else:
         print("Debes programar para el tipo %s aún"%tipo)
-    #
+    
     # Calculamos la matriz jacobiana y su determinante
-    try:
-        j
+    try: j
     except NameError:
         J=dN@X
         if len(J) >1: 
-            j = det(J)
+            j = abs(det(J))
             dN = inv(J)@dN
         else:
-            j=J[0]
+            j = abs(J[0])
             dN = dN/j
     if j<0: 
         print("Cuidado: El jacobiano es negativo!")
@@ -348,4 +443,118 @@ def DofMap(DofNode, connect, NodesElement):
     return dof
 
 
+def Bernoulli(A, x, N, dN, ddN, ProblemData, ElementData, ModelData, dX, tipo):
+    '''Función que retorna la matriz de un EF evaluado en un Punto de Gauss
+    '''
+    n = ElementData.nodes
+    m = ElementData.dof
+    EI = ModelData.EI
+    Area = ModelData.Area
+    ##
+    if tipo == 'MatrizK':
+        A = EI*ddN.T@ddN
+        A = A*dX
+        # print(ddN,A)
+    elif tipo == 'MasaConsistente':
+        rho = ModelData.density
+        A = rho*N.T@N*dX*Area
+        ##
+    elif tipo == 'MasaConcentrada':
+        rho = ModelData.density
+        B = rho*N.T@N*dX*Area
+        ##
+        one = zeros(m*n)
+        one[:] = 1.0
+        B = B@one
+        A = zeros((m*n, m*n))
+        # Concentrando Masas (Mejorar proceso si |A| <= 0)
+        for i in range(m*n):
+            A[i, i] = B[i]
+        ##
+    elif(tipo == 'VectorF'):
+        # Formando matriz N para v
+        N_v = zeros((m*n, 1))
+        N_v = N
+        A = zeros((m*n, 1))
+        f = ModelData.fy
+        A = N_v*f*dX
+        ##
+    else:
+        print("Debes programar para el tipo %s aún" % tipo)
+    return A
 
+
+def Timoshenko(A, x, N, dN, ddN, ProblemData, ElementData, ModelData, dX, tipo):
+    '''Función que retorna la matriz de un EF evaluado en un Punto de Gauss
+    '''
+    n = ElementData.nodes
+    m = ElementData.dof
+    EI = ModelData.EI
+    GAs = ModelData.GAs
+    Area = ModelData.Area
+    ##
+    if tipo == 'MatrizK':
+        if (ProblemData.SpaceDim == 1):
+            # Formando N para theta
+            N_theta = zeros((1, m*n))
+            N_theta[0, 0::m] = N[0]
+            # Formando N para v
+            N_v = zeros((1, m*n))
+            N_v[0, 1::m] = N[0]
+            # Formando B para theta
+            B_theta = zeros((1, m*n))
+            B_theta[0, 0::m] = dN[0]
+            # Formando B para for v
+            B_v = zeros((1, m*n))
+            B_v[0, 1::m] = dN[0]
+            ##
+        elif (ProblemData.SpaceDim == 2):
+            print('Solo es válido para 1D')
+        elif (ProblemData.SpaceDim == 3):
+            print('Solo es válido para 1D')
+        ##
+        # Calculando Matriz
+        A = B_theta.T*EI@B_theta + N_theta.T*GAs@N_theta - N_theta.T*GAs@B_v
+        A = A - B_v.T*GAs@N_theta + B_v.T*GAs@B_v
+        A = A*dX
+    ##
+    elif tipo == 'MasaConsistente':
+        rho = ModelData.density
+        Nmat = zeros((1,4),'float')
+        Nmat[:,1::2] = N
+        A = rho*Nmat.T@Nmat*dX*Area
+        # Artificio para no obetener una Matriz singular
+        A[0, 0] = A[0, 0]+0.01*A[1, 1]
+        A[2, 2] = A[2, 2]+0.01*A[3, 3]
+    ##
+    elif tipo == 'MasaConcentrada':
+        rho = ModelData.density
+        Nmat = zeros((1,4),'float')
+        Nmat[:,1::2] = N
+        B = rho*Nmat.T@Nmat*dX*Area
+        # Artificio para no obetener una Matriz singular
+        B[0, 0] = B[0, 0]+0.01*B[1, 1]
+        B[2, 2] = B[2, 2]+0.01*B[3, 3]
+        ##
+        one = zeros(m*n)
+        one[:] = 1.0
+        B = B@one
+        A = zeros((m*n, m*n))
+        # Concentrando Masas
+        for i in range(m*n):
+            A[i, i] = B[i]
+
+    elif(tipo == 'VectorF'):
+        # Formando matriz N para theta
+        N_theta = zeros((1, m*n))
+        N_theta[0, 0::m] = N[0]
+        # Formando matriz N para v
+        N_v = zeros((1, m*n))
+        N_v[0, 1::m] = N[0]
+        #
+        f = ModelData.fy
+        A = N_v*f*dX
+
+    else:
+        print("Debes programar para el tipo %s aún" % tipo)
+    return A
